@@ -55,7 +55,12 @@ class BaseItem:
 
         self._custom_default_value = custom_default_value
         self._field_type = field_type
-        if isinstance(self._field_type, _GenericAlias):
+        self._default_field_type = field_type
+        if isinstance(self._field_type, (list, tuple)):
+            if len(self._field_type) > 0:
+                self._default_field_type = self._field_type[0]
+
+        elif isinstance(self._field_type, _GenericAlias):
             alias_variable_name: Optional[str] = self._field_type.__dict__.get("_name", None)
             if alias_variable_name is not None:
                 alias_args: Optional[Tuple] = self._field_type.__dict__.get("__args__", None)
@@ -68,6 +73,7 @@ class BaseItem:
                 elif alias_variable_name == "List":
                     raise Exception(f"List not yet implemented.")
 
+
     def validate_data(self, load_data_into_objects: bool) -> Optional[Any]:
         from StructNoSQL.validator import validate_data
         validated_data, valid = validate_data(
@@ -116,6 +122,22 @@ class BaseItem:
         # and return it as the default value.
         return self._field_type()
 
+    @property
+    def custom_default_value(self) -> Optional[Any]:
+        return self._custom_default_value
+
+    @property
+    def value(self) -> Any:
+        return self._value
+
+    @property
+    def field_type(self) -> Any:
+        return self._field_type
+
+    @property
+    def default_field_type(self) -> type:
+        return self._default_field_type
+
     @property
     def dict_key_expected_type(self) -> Optional[type]:
         return self._dict_key_expected_type
@@ -129,6 +151,11 @@ class BaseItem:
         return self._database_path
 
 
+    @property
+    def table(self):
+        return self._table
+
+
 class BaseField(BaseItem):
     def __init__(self, name: str, field_type: Optional[Any] = None, required: Optional[bool] = False, not_modifiable: Optional[bool] = False,
                  custom_default_value: Optional[Any] = None, key_name: Optional[str] = None):
@@ -199,10 +226,6 @@ class BaseField(BaseItem):
     def field_name(self) -> str:
         return self._name
 
-    @property
-    def field_type(self) -> type:
-        return self._field_type
-
     @property
     def key_name(self) -> Optional[str]:
         return self._key_name
@@ -211,18 +234,6 @@ class BaseField(BaseItem):
     def required(self) -> bool:
         return self._required
 
-    @property
-    def value(self) -> Any:
-        return self._value
-
-    @property
-    def database_path(self):
-        return self._database_path
-
-    @property
-    def table(self):
-        return self._table
-
 
 class MapItem(BaseField):
     _default_primitive_type = dict
